/* eslint-disable no-console */
import { Http, Metadata } from '@zeedhi/core';
import {
  AlertService, Date as DataInput, DialogService, LoadingService, Modal, ModalService,
} from '@zeedhi/common';
import { IterableComponentRender } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class ControllerTarefaRecorrente {
  private modalEdicao!: Modal;

  private modalPause!: Modal;

  private modalInicio!: Modal;

  private modalCancelar!: Modal;

  private tarefaSelecionada: any = null;

  public expadCard({ component }: any) {
    component.parent.parent.expanded = !component.parent.parent.expanded;
    component.label = (!component.parent.parent.expanded) ? 'Detalhes' : 'Fechar';
  }

  public async excluirRecorrencia({ component }: any) {
    const form = component!.parent!.parent;
    if (form.validate()) {
      LoadingService.show('Cancelando...');
      await Http.post('/tarefaRecorrente', {
        TIPO: 'CANCELAR',
        NRTAREFARECORRENTE: component.name.split('_')[1],
        DATAINICIAL: Metadata.getInstance<DataInput>('DataInicio').value,
      }).then((response) => {
        if (response.data === 'OK') {
          this.closemodalCancelar();
          Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes').datasource.get();
        } else {
          this.msgDialog('Ocorreu um erro ao tentar encerrar uma tarefa recorrente.<br>Tente novamente mais tarde!<br>Se o erro persistir, contacte o administrador do sistema.', [{
            name: 'cancel',
            component: 'ZdButton',
            outline: true,
            label: 'OK',
            events: {
              click: '{{ControllerTarefaRecorrente.closeDialog}}',
            },
          }]);
        }
      });
      await Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes').datasource.get();
      LoadingService.hide();
    } else {
      this.msgAlert('Preencher campos obrigatorios (*)', 'error');
    }
  }

  public async inicializarRecorrencia({ component }: any) {
    const form = component!.parent!.parent;
    if (form.validate()) {
      LoadingService.show('Inicializando...');
      await Http.post('/recriarTarefasRecorrente', {
        NRTAREFARECORRENTE: component.name.split('_')[1],
        DATAINICIAL: Metadata.getInstance<DataInput>('DataInicio').value,
      }).then((response) => {
        if (response.data === 'OK') {
          this.closemodalInicio();
          Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes').datasource.get();
        } else {
          this.msgDialog('Ocorreu um erro ao tentar iniciar uma tarefa recorrente.<br>Tente novamente mais tarde!<br>Se o erro persistir, contacte o administrador do sistema.', [{
            name: 'cancel',
            component: 'ZdButton',
            outline: true,
            label: 'OK',
            events: {
              click: '{{ControllerTarefaRecorrente.closeDialog}}',
            },
          }]);
        }
      });
      LoadingService.hide();
    } else {
      this.msgAlert('Preencher campos obrigatorios (*)', 'error');
    }
  }

  public async recarregarRecorrencia({ component }: any) {
    LoadingService.show('Recriando...');
    await Http.post('/recriarTarefasRecorrente', {
      NRTAREFARECORRENTE: component.name.split('_')[1],
    }).then((response) => {
      if (response.data === 'OK') {
        Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes').datasource.get();
      } else {
        this.msgDialog('Ocorreu um erro ao tentar recriar as tarefa.<br>Tente novamente mais tarde!<br>Se o erro persistir, contacte o administrador do sistema.', [{
          name: 'cancel',
          component: 'ZdButton',
          outline: true,
          label: 'OK',
          events: {
            click: '{{ControllerTarefaRecorrente.closeDialog}}',
          },
        }]);
      }
    });
    LoadingService.hide();
  }

  public pausarRecorrencia({ component }: any) {
    const form = component!.parent!.parent;
    if (form.validate()) {
      [,this.tarefaSelecionada] = component.name.split('_');
      if (!form.value.DataFinal) {
        this.msgDialog('Pausar sem uma data final fará com que a tarefa fique pausada por tempo indeterminado', null, '{{ControllerTarefaRecorrente.pausar}}');
      } else {
        this.pausar();
      }
    } else {
      this.msgAlert('Preencher campos obrigatorios (*)', 'error');
    }
  }

  private async pausar() {
    LoadingService.show('Pausando...');
    await Http.post('/tarefaRecorrente', {
      TIPO: 'PAUSAR',
      NRTAREFARECORRENTE: this.tarefaSelecionada,
      DATAINICIAL: Metadata.getInstance<DataInput>('DataInicio').value,
      DATAFINAL: Metadata.getInstance<DataInput>('DataFinal').value,
    }).then((response) => {
      if (response.data === 'OK') {
        this.closemodalPause();
        Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes').datasource.get();
      } else {
        this.msgDialog('Ocorreu um erro ao tentar pausar uma tarefa recorrente.<br>Tente novamente mais tarde!<br>Se o erro persistir, contacte o administrador do sistema.', [{
          name: 'cancel',
          component: 'ZdButton',
          outline: true,
          label: 'OK',
          events: {
            click: '{{ControllerTarefaRecorrente.closeDialog}}',
          },
        }]);
      }
    });
    LoadingService.hide();
    this.closeDialog();
  }

  public async editarRecorrencia({ component }: any) {
    const form = component!.parent!.parent;
    if (form.validate() && form.value.checkboxMultipleDate.length > 0) {
      LoadingService.show('Salvando atualização...');
      await Http.post('/tarefaRecorrente', {
        TIPO: 'EDITAR',
        NRTAREFARECORRENTE: this.tarefaSelecionada,
        CDOPERTAREFA: form.value.selectFuncionario,
        CDOPERVISTORIA: form.value.selectEncarregado,
        HORARIO: form.value.selectHoraInicio1,
        DURACAO: (Number(form.value.selectTempoMedio1.split(':')[0]) * 60) + (Number(form.value.selectTempoMedio1.split(':')[1])),
        DIASSEMANA: form.value.checkboxMultipleDate.toString(),
        CDOPERULTATU: localStorage.getItem('CDOPERADOR'),
      }).then(() => {
        this.closeModalEdicao();
        Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes').datasource.get();
        Utilities.showSuccessMessage();
      }).catch((error: any) => {
        Utilities.showErrorMessage(error);
      }).finally(() => {
        LoadingService.hide();
      });
    } else {
      Utilities.alertMessage((form.value.checkboxMultipleDate.length > 0) ? 'Preencher campos obrigatorios (*)' : 'Selecione ao menos um dia da semana.', 'error');
    }
  }

  public visualizacaoIcones({ component }: any) {
    const dadosName = component.name.split('_');
    if (dadosName[0] === 'iconEditar') {
      component.isVisible = (dadosName[2] !== '3');
    }
    if (dadosName[0] === 'iconPause') {
      component.isVisible = (dadosName[2] !== '3' && dadosName[2] === '1');
    }
    if (dadosName[0] === 'iconRecarregar') {
      component.isVisible = (dadosName[2] === '4');
    }
    if (dadosName[0] === 'iconPlay') {
      component.isVisible = (dadosName[2] === '2');
    }
    if (dadosName[0] === 'iconClose') {
      component.isVisible = (dadosName[2] !== '3');
    }
  }

  public corStatus({ component }: any) {
    const dadosName = component.text;
    if (dadosName === 'Em Andamento') {
      component.cssClass = 'conteudo-cards-menores emAndamentoStatus';
    } else if (dadosName === 'Pausada') {
      component.cssClass = 'conteudo-cards-menores pausarStatus';
    } else if (dadosName === 'Cancelada') {
      component.cssClass = 'conteudo-cards-menores caneladoStatus';
    } else if (dadosName === 'Erro na Geração') {
      component.cssClass = 'conteudo-cards-menores erroStatus';
    }
  }

  public async createModalEdicao({ component }: any) {
    const dados = Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes').datasource.data.find((elemento) => elemento.NRTAREFARECORRENTE === Number(component.name.split('_')[1]));
    this.tarefaSelecionada = dados!.NRTAREFARECORRENTE;
    this.modalEdicao = await ModalService.create({
      name: 'modalEdicao',
      persistent: true,
      children: [
        {
          name: 'fromTypescriptHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Tarefas Recorrentes',
              cssClass: 'cor-subtitulos-cards',
              cssStyle: {
                'font-size': '1.5em',
                'margin-bottom': '0.3em',
                'margin-left': '-0.2em',
              },
            },
          ],
          rightSlot: [
            {
              name: 'newPositionCloseBtn',
              component: 'ZdButton',
              iconName: 'mdi-close',
              icon: true,
              events: {
                click: '{{ControllerTarefaRecorrente.closeModalEdicao}}',
              },
            },
          ],
        },
        {
          name: 'form',
          component: 'ZdForm',
          children: [
            {
              name: 'LabelNomeTarefa',
              component: 'ZdText',
              text: 'Tarefa:',
              cssClass: 'cor-subtitulos-cards',
              grid: {
                cols: 4,
              },
            },
            {
              name: 'LabelNomeLocal',
              component: 'ZdText',
              text: 'Local:',
              cssClass: 'cor-subtitulos-cards',
              grid: {
                cols: 4,
              },
            },
            {
              name: 'LabelSituacaoTareda',
              component: 'ZdText',
              text: 'Situação:',
              cssClass: 'cor-subtitulos-cards',
              grid: {
                cols: 4,
              },
            },
            {
              name: 'NomeTarefa',
              component: 'ZdText',
              text: dados!.NMTAREFA,
              grid: {
                cols: 4,
              },
            },
            {
              name: 'NomeLocal',
              component: 'ZdText',
              text: dados!.NMLOCALCLIENTE,
              grid: {
                cols: 4,
              },
            },
            {
              name: 'SituacaoTareda',
              component: 'ZdText',
              text: dados!.NMIDSITUACAO,
              grid: {
                cols: 4,
              },
            },
            {
              name: 'selectHoraInicio1',
              component: 'ZdTime',
              autocomplete: false,
              label: 'Inicio',
              timeFormat: '24hr',
              placeholder: '00:00',
              mask: '##:##',
              value: dados!.HORARIO,
              validations: {
                required: {},
              },
              grid: {
                cols: 6,
              },
            },
            {
              name: 'selectTempoMedio1',
              component: 'ZdTextInput',
              label: 'Duração (minutos)',
              placeholder: '00:00',
              mask: '##:M#',
              value: `${String(Math.trunc(Number(dados!.DURACAO || 0) / 60)).padStart(2, '0')}:${String(Math.trunc(Number(dados!.DURACAO || 0) % 60)).padStart(2, '0')}`,
              validations: {
                required: {
                  message: 'Campo Obrigatorio',
                },
                minLength: {
                  limit: 5,
                  message: 'Campo Obrigatorio',
                },
                maxLength: {
                  limit: 5,
                  message: 'Campo Obrigatorio',
                },
              },
              grid: {
                cols: 6,
              },
            },
            {
              name: 'checkboxMultipleDate',
              component: 'ZdCheckboxMultiple',
              columns: 7,
              vertical: false,
              dataLabel: 'value',
              dataValue: 'id',
              value: (dados!.DIASSEMANA.split(',').map(Number) || null),
              grid: {
                cols: 12,
              },
              cssStyle: {
                'font-size': '14px !import',
              },
              datasource: {
                data: [
                  {
                    id: 1,
                    value: 'Segunda-Feira',
                  },
                  {
                    id: 2,
                    value: 'Terça-Feira',
                  },
                  {
                    id: 3,
                    value: 'Quarta-Feira',
                  },
                  {
                    id: 4,
                    value: 'Quinta-Feira',
                  },
                  {
                    id: 5,
                    value: 'Sexta-Feira',
                  },
                  {
                    id: 6,
                    value: 'Sabado',
                  },
                  {
                    id: 0,
                    value: 'Domingo',
                  },
                ],
              },
            },
            {
              name: 'selectFuncionario',
              component: 'ZdSelect',
              autocomplete: true,
              label: 'Funcionário',
              dataText: 'NMOPERADOR',
              dataValue: 'CDOPERADOR',
              value: (dados!.CDOPERTAREFA || null),
              cssClass: 'zd-mt-4',
              datasource: {
                route: '/findOperador',
                type: 'rest',
                lazyLoad: false,
              },
              grid: {
                cols: 6,
              },
            },
            {
              name: 'selectEncarregado',
              component: 'ZdSelect',
              autocomplete: true,
              label: 'Encarregado',
              dataText: 'NMOPERADOR',
              dataValue: 'CDOPERADOR',
              value: (dados!.CDOPERVISTORIA || null),
              cssClass: 'zd-mt-4',
              datasource: {
                route: '/findOperador',
                type: 'rest',
                lazyLoad: false,
              },
              grid: {
                cols: 6,
              },
            },
            {
              name: 'linhaComBtnsDeEditarTarefaRecorrente',
              component: 'ZdRow',
              cssClass: 'zd-mt-3',
              grid: {
                cols: 12,
              },
              justify: 'end',
              children: [
                {
                  name: 'BtnCancelar',
                  component: 'ZdButton',
                  label: 'Cancelar',
                  outline: true,
                  cssClass: 'zd-mx-4',
                  width: '100px',
                  events: {
                    click: '{{ControllerTarefaRecorrente.closeModalEdicao}}',
                  },
                },
                {
                  name: 'BtnSalvar',
                  component: 'ZdButton',
                  label: 'Salvar',
                  width: '100px',
                  events: {
                    click: '{{ControllerTarefaRecorrente.editarRecorrencia}}',
                  },
                },
              ],
            },
          ],
        },
      ],
    });
    this.modalEdicao.show();
  }

  public closeModalEdicao() {
    this.modalEdicao.hide();
    this.tarefaSelecionada = null;
    this.modalEdicao?.destroy();
  }

  public async createmodalPause({ component }: any) {
    const dados = Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes').datasource.data.find((elemento) => elemento.NRTAREFARECORRENTE === Number(component.name.split('_')[1]));
    this.tarefaSelecionada = dados!.NRTAREFARECORRENTE;
    this.modalPause = await ModalService.create({
      name: 'modalPause',
      persistent: true,
      children: [
        {
          name: 'fromTypescriptHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Pausar Recorrência',
              cssStyle: {
                'font-size': '1.5em',
                'margin-bottom': '0.3em',
                'margin-left': '-0.2em',
              },
            },
          ],
          rightSlot: [
            {
              name: 'newPositionCloseBtn',
              component: 'ZdButton',
              iconName: 'mdi-close',
              icon: true,
              events: {
                click: '{{ControllerTarefaRecorrente.closemodalPause}}',
              },
            },
          ],
        },
        {
          name: 'form',
          component: 'ZdForm',
          children: [
            {
              name: 'DataInicio',
              component: 'ZdDate',
              label: 'Inicio',
              displayFormat: 'DD/MM/YYYY',
              dateFormat: 'DD/MM/YYYY',
              grid: {
                cols: 6,
              },
              validations: {
                required: {},
              },
            },
            {
              name: 'DataFinal',
              component: 'ZdDate',
              label: 'Fim',
              displayFormat: 'DD/MM/YYYY',
              dateFormat: 'DD/MM/YYYY',
              grid: {
                cols: 6,
              },
            },
            {
              name: 'linhaComBtnsDeEditarTarefaRecorrente',
              component: 'ZdRow',
              cssClass: 'zd-mt-3',
              grid: {
                cols: 12,
              },
              justify: 'end',
              children: [
                {
                  name: 'BtnCancelar',
                  component: 'ZdButton',
                  label: 'Cancelar',
                  outline: true,
                  cssClass: 'zd-mx-4',
                  width: '100px',
                  events: {
                    click: '{{ControllerTarefaRecorrente.closemodalPause}}',
                  },
                },
                {
                  name: `BtnSalvar_${component.name.split('_')[1]}`,
                  component: 'ZdButton',
                  label: 'Salvar',
                  width: '100px',
                  events: {
                    click: '{{ControllerTarefaRecorrente.pausarRecorrencia}}',
                  },
                },
              ],
            },
          ],
        },
      ],
    });
    this.modalPause.show();
  }

  public closemodalPause() {
    this.modalPause.hide();
    this.tarefaSelecionada = null;
    this.modalPause?.destroy();
  }

  public async createmodalInicio({ component }: any) {
    const dados = Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes').datasource.data.find((elemento) => elemento.NRTAREFARECORRENTE === Number(component.name.split('_')[1]));
    this.tarefaSelecionada = dados!.NRTAREFARECORRENTE;
    this.modalInicio = await ModalService.create({
      name: 'modalInicio',
      persistent: true,
      children: [
        {
          name: 'fromTypescriptHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Iniciar Recorrência',
              cssStyle: {
                'font-size': '1.5em',
                'margin-bottom': '0.3em',
                'margin-left': '-0.2em',
              },
            },
          ],
          rightSlot: [
            {
              name: 'newPositionCloseBtn',
              component: 'ZdButton',
              iconName: 'mdi-close',
              icon: true,
              events: {
                click: '{{ControllerTarefaRecorrente.closemodalInicio}}',
              },
            },
          ],
        },
        {
          name: 'form',
          component: 'ZdForm',
          children: [
            {
              name: 'DataInicio',
              component: 'ZdDate',
              label: 'Inicio',
              displayFormat: 'DD/MM/YYYY',
              dateFormat: 'DD/MM/YYYY',
              grid: {
                cols: 12,
              },
              validations: {
                required: {},
              },
            },
            {
              name: 'linhaComBtnsDeEditarTarefaRecorrente',
              component: 'ZdRow',
              cssClass: 'zd-mt-3',
              grid: {
                cols: 12,
              },
              justify: 'end',
              children: [
                {
                  name: 'BtnCancelar',
                  component: 'ZdButton',
                  label: 'Cancelar',
                  outline: true,
                  cssClass: 'zd-mx-4',
                  width: '100px',
                  events: {
                    click: '{{ControllerTarefaRecorrente.closemodalInicio}}',
                  },
                },
                {
                  name: `BtnSalvar_${component.name.split('_')[1]}`,
                  component: 'ZdButton',
                  label: 'Salvar',
                  width: '100px',
                  events: {
                    click: '{{ControllerTarefaRecorrente.inicializarRecorrencia}}',
                  },
                },
              ],
            },
          ],
        },
      ],
    });
    this.modalInicio.show();
  }

  public closemodalInicio() {
    this.modalInicio.hide();
    this.tarefaSelecionada = null;
    this.modalInicio?.destroy();
  }

  public async createmodalCancelar({ component }: any) {
    const dados = Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes').datasource.data.find((elemento) => elemento.NRTAREFARECORRENTE === Number(component.name.split('_')[1]));
    this.tarefaSelecionada = dados!.NRTAREFARECORRENTE;
    this.modalCancelar = await ModalService.create({
      persistent: true,
      name: 'modalCancelar',
      children: [
        {
          name: 'fromTypescriptHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Cancelar Recorrência',
              cssStyle: {
                'font-size': '1.5em',
                'margin-bottom': '0.3em',
                'margin-left': '-0.2em',
              },
            },
          ],
          rightSlot: [
            {
              name: 'newPositionCloseBtn',
              component: 'ZdButton',
              iconName: 'mdi-close',
              icon: true,
              events: {
                click: '{{ControllerTarefaRecorrente.closemodalCancelar}}',
              },
            },
          ],
        },
        {
          name: 'form',
          component: 'ZdForm',
          children: [
            {
              name: 'DataInicio',
              component: 'ZdDate',
              label: 'Inicio',
              displayFormat: 'DD/MM/YYYY',
              grid: {
                cols: 12,
              },
              dateFormat: 'DD/MM/YYYY',
              validations: {
                required: {},
              },
            },
            {
              name: 'linhaComBtnsDeEditarTarefaRecorrente',
              component: 'ZdRow',
              cssClass: 'zd-mt-3',
              grid: {
                cols: 12,
              },
              justify: 'end',
              children: [
                {
                  name: 'BtnCancelar',
                  component: 'ZdButton',
                  label: 'Cancelar',
                  outline: true,
                  cssClass: 'zd-mx-4',
                  width: '100px',
                  events: {
                    click: '{{ControllerTarefaRecorrente.closemodalCancelar}}',
                  },
                },
                {
                  name: `BtnSalvar_${component.name.split('_')[1]}`,
                  component: 'ZdButton',
                  label: 'Salvar',
                  width: '100px',
                  events: {
                    click: '{{ControllerTarefaRecorrente.excluirRecorrencia}}',
                  },
                },
              ],
            },
          ],
        },
      ],
    });
    this.modalCancelar.show();
  }

  public closemodalCancelar() {
    this.modalCancelar.hide();
    this.tarefaSelecionada = null;
    this.modalCancelar?.destroy();
  }

  public msgAlert(mensagem: string = 'Alert', tipo: string = 'sucess') {
    AlertService.show({
      name: 'basic-alert',
      text: mensagem,
      top: true,
      left: false,
      timeout: 3000,
      color: tipo,
    });
  }

  public msgDialog(texto: string, buttons: any, rota: string = '{{ControllerTarefaRecorrente.msgAlert}}') {
    const btnsPadrão = [{
      name: 'cancel',
      component: 'ZdButton',
      outline: true,
      label: 'Voltar',
      events: {
        click: '{{ControllerTarefaRecorrente.closeDialog}}',
      },
    },
    {
      name: 'yes',
      component: 'ZdButton',
      label: 'Salvar',
      events: {
        click: rota,
      },
    }];
    const btns = buttons || btnsPadrão;

    DialogService.show({
      name: 'basic-dialog',
      type: 'warning',
      title: 'Atenção!',
      text: `${texto}`,
      buttons: btns,
    });
  }

  private closeDialog() {
    this.tarefaSelecionada = null;
    DialogService.hide();
  }

  public filterIterable({ component }:any) {
    const iterable = Metadata.getInstance<IterableComponentRender>('IteracaoTarefasRecorrentes');
    iterable.datasource.addFilter(component.dataValue, component.value);
  }
}
