import { Utilities } from '@/util/Utilities';
import {
    Button,
    Form, Modal, ModalService, Select, SelectMultiple, TextInput,
} from '@zeedhi/common';
import { Metadata } from '@zeedhi/core';
import { TekTreeGrid } from '@zeedhi/teknisa-components-common';

export class Fm00003000ChecklistController {
    public modalChecklist!: Modal;

    public visibilidadeGrid: boolean = false;

    public visibilidadeBotaoGerar: boolean = !this.visibilidadeGrid;

    public check: boolean = false;

    public async abrirModal() {
        this.modalChecklist = await ModalService.createFromJsonPost('', { jsonPath: 'fm_00003_000_checklist/modal/fm_00003_001_salvar_checklist' });
        this.modalChecklist.show();
    }

    public fecharModal() {
        this.modalChecklist.hide();
        this.modalChecklist?.destroy();
    }

    public gerarDadosChecklist() {
        const form = Metadata.getInstance<Form>('form-checklist');
        const treeGrid = Metadata.getInstance<TekTreeGrid>('tree-grid-checklist');
        const selectAtividades = Metadata.getInstance<Select>('atividades-checklist');

        if (Utilities.validateForm(form)) {
            this.visibilidadeGrid = true;
            treeGrid.datasource.setFilter({
                NRTAREFA: selectAtividades.value,
            });
        }
    }

    public changeAtividades({ component }: any) {
        const selectAtividades = Metadata.getInstance<Select>('atividades-checklist');

        if (component.value.length > 0) {
            selectAtividades.disabled = false;
            selectAtividades.datasource.setFilter({
                NRATIVIDADE: component.value,
            });
        } else {
            selectAtividades.disabled = true;
            selectAtividades.value = '';
            this.visibilidadeGrid = false;
        }
    }

    public changeVisibilidadeGrid({ component }: any) {
        if (component.value.length > 0) {
            this.visibilidadeGrid = false;
        }
    }

    public habilitarBotaoSalvar() {
        Metadata.getInstance<Button>('salvar-checklist').disabled = true;
    }

    public salvarChecklist() {
        const grid = Metadata.getInstance<TekTreeGrid>('tree-grid-checklist');
        const tekGrid = Metadata.getInstance<TekTreeGrid>('tek-grid-checklist');

        if (grid.selectedRows.length > 0) {
            const escopos = (grid.selectedRows.filter((elemento: any) => elemento.TIPO === 'Atividade')).map((item: any) => ({ NRTAREFA: item.CODIGO, NRESCOPO: item.CODIGOPAI }));
            const criterios = (grid.selectedRows.filter((elemento: any) => elemento.TIPO === 'Critério')).map((item: any) => ({ NRCRITERIO: item.CODIGO, NRTAREFA: item.CODIGOPAI }));

            Utilities.save('/salvarChecklist', {
                ESCOPOS: escopos,
                CRITERIOS: criterios,
                NMCHECKLIST: Metadata.getInstance<TextInput>('nome-checklist').value,
                NRTIPOLOCALCLI: Metadata.getInstance<Select>('tipo-local-checklist').value,
                NRATIVIDADE: Metadata.getInstance<SelectMultiple>('itens-checklist').value,
                NRTAREFA: Metadata.getInstance<SelectMultiple>('atividades-checklist').value,
            }).then(() => {
                this.fecharModal();
                tekGrid.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            });
        }
    }
}
